/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Community System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.core.impl;

import java.sql.*;
import java.util.*;
import org.apache.log4j.*;
import com.silverwrist.venice.db.*;
import com.silverwrist.venice.security.AuditRecord;
import com.silverwrist.venice.core.*;

class TopicUserContextImpl implements TopicContext
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  private static Category logger = Category.getInstance(TopicUserContextImpl.class.getName());

  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private EngineBackend engine;
  private ConferenceBackend conf;
  private DataPool datapool;
  private int topicid;
  private short topicnum;
  private int creator_uid;
  private int top_message;
  private boolean frozen;
  private boolean archived;
  private java.util.Date created;
  private java.util.Date lastupdate;
  private String name;
  private boolean hidden;
  private int unread;
  private boolean deleted = false;

  /*--------------------------------------------------------------------------------
   * Constructors
   *--------------------------------------------------------------------------------
   */

  protected TopicUserContextImpl(EngineBackend engine, ConferenceBackend conf, DataPool datapool, int topicid,
				 short topicnum, int creator_uid, int top_message, boolean frozen,
				 boolean archived, java.util.Date created, java.util.Date lastupdate,
				 String name, boolean hidden, int unread)
  {
    this.engine = engine;
    this.conf = conf;
    this.datapool = datapool;
    this.topicid = topicid;
    this.topicnum = topicnum;
    this.creator_uid = creator_uid;
    this.top_message = top_message;
    this.frozen = frozen;
    this.archived = archived;
    this.created = created;
    this.lastupdate = lastupdate;
    this.name = name;
    this.hidden = hidden;
    this.unread = unread;

  } // end TopicUserContextImpl

  /*--------------------------------------------------------------------------------
   * Internal functions
   *--------------------------------------------------------------------------------
   */

  private static ResultSet queryByTopic(Statement stmt, int topicid, int uid) throws SQLException
  {
    StringBuffer sql =
      new StringBuffer("SELECT t.topicid, t.num, t.creator_uid, t.top_message, t.frozen, t.archived, "
		       + "t.createdate, t.lastupdate, t.name, IFNULL(s.hidden,0) AS hidden, "
		       + "(t.top_message - IFNULL(s.last_message,-1)) AS unread FROM topics t "
		       + "LEFT JOIN topicsettings s ON t.topicid = s.topicid AND s.uid = ");
    sql.append(uid).append(" WHERE t.topicid = ").append(topicid).append(';');
    if (logger.isDebugEnabled())
      logger.debug("SQL: " + sql.toString());
    return stmt.executeQuery(sql.toString());

  } // end queryByTopic

  private void makeDeleted()
  {
    top_message = -1;
    frozen = false;
    archived = false;
    created = null;
    lastupdate = null;
    name = null;
    hidden = false;
    unread = -1;
    deleted = true;

  } // end if

  /*--------------------------------------------------------------------------------
   * Implementatuions from interface TopicContext
   *--------------------------------------------------------------------------------
   */

  public void refresh() throws DataException
  {
    if (logger.isDebugEnabled())
      logger.debug("refreshing topic ID " + String.valueOf(topicid));
    Connection conn = null;      // pooled database connection

    try
    { // get a database connection
      conn = datapool.getConnection();
      Statement stmt = conn.createStatement();

      // perform a requery of the database
      ResultSet rs = queryByTopic(stmt,topicid,conf.realUID());
      if (rs.next())
      { // update the fields that are capable of changing
	top_message = rs.getInt(4);
	frozen = rs.getBoolean(5);
	archived = rs.getBoolean(6);
	lastupdate = SQLUtil.getFullDateTime(rs,8);
	hidden = rs.getBoolean(10);
	unread = rs.getInt(11);

      } // end if
      else // this topic must have been deleted - fsck it
	makeDeleted();

    } // end try
    catch (SQLException e)
    { // turn SQLException into data exception
      logger.error("DB error reading topic data: " + e.getMessage(),e);
      throw new DataException("unable to retrieve topic information: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure we release the connection before we go
      if (conn!=null)
	datapool.releaseConnection(conn);

    } // end finally

  } // end refresh

  public int getTopicID()
  {
    return topicid;

  } // end getTopicID

  public short getTopicNumber()
  {
    return topicnum;

  } // end getTopicNumber

  public String getName()
  {
    return name;

  } // end getName

  public int getUnreadMessages()
  {
    return unread;

  } // end getUnreadMessages

  public int getTotalMessages()
  {
    return top_message + 1;

  } // end getTotalMessages

  public java.util.Date getLastUpdateDate()
  {
    return lastupdate;

  } // end getLastUpdateDate

  public int getCreatorUID()
  {
    return creator_uid;

  } // end getCreatorUID

  public boolean isFrozen()
  {
    return frozen;

  } // end isFrozen

  public boolean isArchived()
  {
    return archived;

  } // end isArchived

  public java.util.Date getCreatedDate()
  {
    return created;

  } // end getCreatedDate

  public boolean isHidden()
  {
    return hidden;

  } // end isHidden

  public boolean isDeleted()
  {
    return deleted;

  } // end isDeleted

  public boolean canFreeze()
  {
    return conf.userCanHide();

  } // end canFreeze

  public boolean canArchive()
  {
    return conf.userCanHide();

  } // end canArchive

  public void setFrozen(boolean flag) throws DataException, AccessError
  {
    if ((frozen==flag) || deleted)
      return;  // no-op
    if (!(conf.userCanHide()))
    { // you can't freeze the topic!
      logger.error("user cannot change frozen status of topic");
      throw new AccessError("You are not permitted to freeze or unfreeze this topic.");

    } // end if

    Connection conn = null;      // pooled database connection
    AuditRecord ar = null;       // audit record indicating success

    try
    { // get a database connection
      conn = datapool.getConnection();
      Statement stmt = conn.createStatement();

      // create the SQL statement to freeze or unfreeze the topic
      StringBuffer sql = new StringBuffer("UPDATE topics SET frozen = ");
      sql.append(flag ? '1' : '0').append(" WHERE topicid = ").append(topicid).append(';');
      if (stmt.executeUpdate(sql.toString())>0)
      { // success! save the flag and generate an audit record
	frozen = flag;
	ar = new AuditRecord(AuditRecord.TOPIC_FREEZE,conf.realUID(),conf.userRemoteAddress(),
			     conf.realSIGID(),"conf=" + String.valueOf(conf.realConfID()) + ",topic="
			     + String.valueOf(topicid),flag ? "freeze" : "unfreeze");

      } // end if
      else  // somebody else must have deleted this topic
	makeDeleted();

    } // end try
    catch (SQLException e)
    { // turn SQLException into data exception
      logger.error("DB error setting topic data: " + e.getMessage(),e);
      throw new DataException("unable to set topic frozen status: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure we release the connection before we go
      try
      { // save off the audit record before we go, though
	if ((ar!=null) && (conn!=null))
	  ar.store(conn);

      } // end try
      catch (SQLException e)
      { // we couldn't store the audit record!
	logger.error("DB error saving audit record: " + e.getMessage(),e);

      } // end catch

      if (conn!=null)
	datapool.releaseConnection(conn);

    } // end finally

  } // end setFrozen

  public void setArchived(boolean flag) throws DataException, AccessError
  {
    if ((archived==flag) || deleted)
      return;  // no-op
    if (!(conf.userCanHide()))
    { // you can't archive the topic!
      logger.error("user cannot change archived status of topic");
      throw new AccessError("You are not permitted to archive or unarchive this topic.");

    } // end if

    Connection conn = null;      // pooled database connection
    AuditRecord ar = null;       // audit record indicating success

    try
    { // get a database connection
      conn = datapool.getConnection();
      Statement stmt = conn.createStatement();

      // create the SQL statement to freeze or unfreeze the topic
      StringBuffer sql = new StringBuffer("UPDATE topics SET archived = ");
      sql.append(flag ? '1' : '0').append(" WHERE topicid = ").append(topicid).append(';');
      if (stmt.executeUpdate(sql.toString())>0)
      { // success! save the flag and generate an audit record
	archived = flag;
	ar = new AuditRecord(AuditRecord.TOPIC_ARCHIVE,conf.realUID(),conf.userRemoteAddress(),
			     conf.realSIGID(),"conf=" + String.valueOf(conf.realConfID()) + ",topic="
			     + String.valueOf(topicid),flag ? "archive" : "unarchive");

      } // end if
      else  // somebody else must have deleted this topic
	makeDeleted();

    } // end try
    catch (SQLException e)
    { // turn SQLException into data exception
      logger.error("DB error setting topic data: " + e.getMessage(),e);
      throw new DataException("unable to set topic archived status: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure we release the connection before we go
      try
      { // save off the audit record before we go, though
	if ((ar!=null) && (conn!=null))
	  ar.store(conn);

      } // end try
      catch (SQLException e)
      { // we couldn't store the audit record!
	logger.error("DB error saving audit record: " + e.getMessage(),e);

      } // end catch

      if (conn!=null)
	datapool.releaseConnection(conn);

    } // end finally

  } // end setArchived

  public void setHidden(boolean flag) throws DataException
  {
    if ((hidden==flag) || deleted)
      return;  // no-op

    Connection conn = null;      // pooled database connection

    try
    { // get a database connection
      conn = datapool.getConnection();
      Statement stmt = conn.createStatement();
      stmt.executeUpdate("LOCK TABLES topicsettings WRITE, topics READ;");

      try
      { // start by trying to see if we can update topicsettings directly
	StringBuffer sql = new StringBuffer("UPDATE topicsettings SET hidden = ");
	sql.append(flag ? '1' : '0').append(" WHERE topicid = ").append(topicid).append(" AND uid = ");
	sql.append(conf.realUID()).append(';');
	if (stmt.executeUpdate(sql.toString())>0)
	{ // that was all we needed - just save the flag and exit
	  hidden = flag;
	  return;

	} // end if

	// OK, check: Is the topic still there?!?
	sql.setLength(0);
	sql.append("SELECT topicid from topics WHERE topicid = ").append(topicid).append(';');
	ResultSet rs = stmt.executeQuery(sql.toString());
	if (!(rs.next()))
	{ // the topic's been deleted - bail out
	  makeDeleted();
	  return;

	} // end if

	// OK, just insert a new row into topicsettings, why dontcha...
	sql.setLength(0);
	sql.append("INSERT INTO topicsettings (topicid, uid, hidden) VALUES (").append(topicid).append(", ");
	sql.append(conf.realUID()).append(", ").append(flag ? '1' : '0').append(");");
	stmt.executeUpdate(sql.toString());
	hidden = flag;  // successful completion

      } // end try
      finally
      { // unlock the tables before we go
	Statement ulk_stmt = conn.createStatement();
	ulk_stmt.executeUpdate("UNLOCK TABLES;");

      } // end finally

    } // end try
    catch (SQLException e)
    { // turn SQLException into data exception
      logger.error("DB error setting topic data: " + e.getMessage(),e);
      throw new DataException("unable to set topic hidden status: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure we release the connection before we go
      if (conn!=null)
	datapool.releaseConnection(conn);

    } // end finally

  } // end setHidden

  public int getFirstUnreadMessage()
  {
    if (unread==0)
      return -1;
    else
      return top_message - (unread - 1);

  } // end getFirstUnreadMessage

  public void setUnreadMessages(int count) throws DataException
  {
    if (count>(top_message+1))   // constrain count to [0, top_message+1]
      count = top_message + 1;
    else if (count<0)
      count = 0;
    if ((count==unread) || deleted)
      return; // no-op

    int last_msg = top_message - count;
    Connection conn = null;      // pooled database connection

    try
    { // get a database connection
      conn = datapool.getConnection();
      Statement stmt = conn.createStatement();
      stmt.executeUpdate("LOCK TABLES confsettings WRITE, topicsettings WRITE, topics READ;");

      try
      { // start by trying to see if we can update topicsettings directly
	StringBuffer sql = new StringBuffer("UPDATE topicsettings SET last_message = ");
	sql.append(last_msg).append(" WHERE topicid = ").append(topicid).append(" AND uid = ");
	sql.append(conf.realUID()).append(';');
	if (stmt.executeUpdate(sql.toString())>0)
	{ // that was all we needed - just save the flag and exit
	  conf.touchRead(conn);
	  unread = count;
	  return;

	} // end if

	// OK, check: Is the topic still there?!?
	sql.setLength(0);
	sql.append("SELECT topicid from topics WHERE topicid = ").append(topicid).append(';');
	ResultSet rs = stmt.executeQuery(sql.toString());
	if (!(rs.next()))
	{ // the topic's been deleted - bail out
	  makeDeleted();
	  return;

	} // end if

	// OK, just insert a new row into topicsettings, why dontcha...
	sql.setLength(0);
	sql.append("INSERT INTO topicsettings (topicid, uid, last_message) VALUES (").append(topicid);
	sql.append(", ").append(conf.realUID()).append(", ").append(last_msg).append(");");
	stmt.executeUpdate(sql.toString());
	conf.touchRead(conn);
	unread = count;  // successful completion

      } // end try
      finally
      { // unlock the tables before we go
	Statement ulk_stmt = conn.createStatement();
	ulk_stmt.executeUpdate("UNLOCK TABLES;");

      } // end finally

    } // end try
    catch (SQLException e)
    { // turn SQLException into data exception
      logger.error("DB error setting topic data: " + e.getMessage(),e);
      throw new DataException("unable to set topic hidden status: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure we release the connection before we go
      if (conn!=null)
	datapool.releaseConnection(conn);

    } // end finally

  } // end setUnreadMessages

  public void fixSeen() throws DataException
  {
    setUnreadMessages(0);

  } // end fixSeen

  /*--------------------------------------------------------------------------------
   * External operations usable only from within the package
   *--------------------------------------------------------------------------------
   */

  static List getTopicList(EngineBackend engine, ConferenceBackend conf, DataPool datapool, int get_option,
			   int sort_option) throws DataException
  {
    if (logger.isDebugEnabled())
      logger.debug("getTopicList for conf # " + String.valueOf(conf.realConfID()) + ", user #"
		   + String.valueOf(conf.realUID()));
    Vector rc = new Vector();    // return from this function
    Connection conn = null;      // pooled database connection

    try
    { // get a database connection
      conn = datapool.getConnection();
      Statement stmt = conn.createStatement();

      // Figure out what the "where" clause of the SQL statement will be.  This is in addition to a
      // WHERE t.confid = (confid) that is always present.  (Some of these WHERE specifiers refer to
      // computed fields of the SELECT.)
      String where_clause;
      switch (get_option)
      {
	case ConferenceContext.GET_ALL:
	  where_clause = null;   // no filtering options
	  break;

	case ConferenceContext.DISPLAY_NEW:
	  where_clause = "hidden = 0 AND unread > 0";  // only non-hidden topics w/unread messages
	  break;

	case ConferenceContext.DISPLAY_ACTIVE:
	  where_clause = "t.archived = 0 AND hidden = 0";  // only non-hidden, non-archived topics
	  break;

	case ConferenceContext.DISPLAY_ALL:
	  where_clause = "t.archived = 0 AND hidden = 0";  // only non-hidden, non-archived topics
	  break;

	case ConferenceContext.DISPLAY_HIDDEN:
	  where_clause = "hidden = 1";  // only hidden topics
	  break;

	case ConferenceContext.DISPLAY_ARCHIVED:
	  where_clause = "t.archived = 1 AND hidden = 0";  // only non-hidden, archived topics
	  break;

	default:
	  logger.error("ACK! get_option (" + String.valueOf(get_option) + ") is bogus!");
	  throw new DataException("Invalid retrieval option specified.");

      } // end switch

      // Figure out what the "order by" clause of the SQL statement will be.  In the case of
      // DISPLAY_NEW mode, this will be in addition to an ordering clause that puts all topics
      // with unread messages first.
      boolean reverse = false;
      int real_sort_option = sort_option;
      if (sort_option<0)
      { // did we specify a "reverse" option?
	reverse = true;
	real_sort_option = -sort_option;

      } // end if

      String order_by_clause;
      switch (real_sort_option)
      {
	case ConferenceContext.SORT_TOPICID:
	  // no reverse possible here
	  order_by_clause = "t.topicid ASC";
	  break;

	case ConferenceContext.SORT_NUMBER:
	  if (reverse)
	    order_by_clause = "t.num DESC";
	  else
	    order_by_clause = "t.num ASC";
	  break;

	case ConferenceContext.SORT_NAME:
	  if (reverse)
	    order_by_clause = "t.name DESC, t.num DESC";
	  else
	    order_by_clause = "t.name ASC, t.num ASC";
	  break;

	case ConferenceContext.SORT_UNREAD:
	  if (reverse)
	    order_by_clause = "unread ASC, t.num DESC";
	  else
	    order_by_clause = "unread DESC, t.num ASC";
	  break;

	case ConferenceContext.SORT_TOTAL:
	  if (reverse)
	    order_by_clause = "t.top_message ASC, t.num DESC";
	  else
	    order_by_clause = "t.top_message DESC, t.num ASC";
	  break;

	case ConferenceContext.SORT_DATE:
	  if (reverse)
	    order_by_clause = "t.lastupdate ASC, t.num DESC";
	  else
	    order_by_clause = "t.lastupdate DESC, t.num ASC";
	  break;

	default:
	  logger.error("ACK! sort_option (" + String.valueOf(sort_option) + ") is bogus!");
	  throw new DataException("Invalid sort option specified.");

      } // end switch

      // Build the huge SQL statement to feed to the database.
      StringBuffer sql =
	new StringBuffer("SELECT t.topicid, t.num, t.creator_uid, t.top_message, t.frozen, t.archived, "
			 + "t.createdate, t.lastupdate, t.name, IFNULL(s.hidden,0) AS hidden, "
			 + "(t.top_message - IFNULL(s.last_message,-1)) AS unread");
      if (get_option==ConferenceContext.DISPLAY_ACTIVE)
	sql.append(", SIGN(t.top_message - IFNULL(s.last_message,-1)) AS newflag");
      sql.append(" FROM topics t LEFT JOIN topicsettings s ON t.topicid = s.topicid AND s.uid = ");
      sql.append(conf.realUID()).append(" WHERE t.confid = ").append(conf.realConfID());
      if (where_clause!=null)
	sql.append(" AND ").append(where_clause);
      sql.append(" ORDER BY ");
      if (get_option==ConferenceContext.DISPLAY_ACTIVE)
	sql.append("newflag DESC, ");
      sql.append(order_by_clause).append(';');
      if (logger.isDebugEnabled())
	logger.debug("SQL: " + sql.toString());

      // Now pass this query off to the database!
      ResultSet rs = stmt.executeQuery(sql.toString());

      while (rs.next())
      { // create the returned objects and add them to the vector
	TopicContext top =
	    new TopicUserContextImpl(engine,conf,datapool,rs.getInt(1),rs.getShort(2),rs.getInt(3),
				     rs.getInt(4),rs.getBoolean(5),rs.getBoolean(6),
				     SQLUtil.getFullDateTime(rs,7),SQLUtil.getFullDateTime(rs,8),
				     rs.getString(9),rs.getBoolean(10),rs.getInt(11));
	rc.add(top);

      } // end while

    } // end try
    catch (SQLException e)
    { // turn SQLException into data exception
      logger.error("DB error reading topic entries: " + e.getMessage(),e);
      throw new DataException("unable to retrieve topic information: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure we release the connection before we go
      if (conn!=null)
	datapool.releaseConnection(conn);

    } // end finally

    return new ReadOnlyVector(rc);

  } // end getTopicList

  static TopicContext getTopicByID(EngineBackend engine, ConferenceBackend conf, DataPool datapool,
				   int topicid) throws DataException
  {
    if (logger.isDebugEnabled())
      logger.debug("getTopicByID for topic # " + String.valueOf(topicid) + ", user #"
		   + String.valueOf(conf.realUID()));

    Connection conn = null;      // pooled database connection

    try
    { // get a database connection
      conn = datapool.getConnection();
      Statement stmt = conn.createStatement();

      // Query the dtabase by topic ID.
      ResultSet rs = queryByTopic(stmt,topicid,conf.realUID());
      if (rs.next()) // found it!
	return new TopicUserContextImpl(engine,conf,datapool,topicid,rs.getShort(2),rs.getInt(3),
					rs.getInt(4),rs.getBoolean(5),rs.getBoolean(6),
					SQLUtil.getFullDateTime(rs,7),SQLUtil.getFullDateTime(rs,8),
					rs.getString(9),rs.getBoolean(10),rs.getInt(11));
      // else fall out and return null

    } // end try
    catch (SQLException e)
    { // turn SQLException into data exception
      logger.error("DB error reading topic entries: " + e.getMessage(),e);
      throw new DataException("unable to retrieve topic information: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure we release the connection before we go
      if (conn!=null)
	datapool.releaseConnection(conn);

    } // end finally

    return null;  // default return

  } // end getTopicByID

  static TopicContext getTopicByNumber(EngineBackend engine, ConferenceBackend conf, DataPool datapool,
				       short topicnum) throws DataException
  {
    if (logger.isDebugEnabled())
      logger.debug("getTopicByNumber for topic # " + String.valueOf(topicnum) + ", conf # "
		   + String.valueOf(conf.realConfID()) + ", user #" + String.valueOf(conf.realUID()));

    Connection conn = null;      // pooled database connection

    try
    { // get a database connection
      conn = datapool.getConnection();
      Statement stmt = conn.createStatement();

      // Build the database query.
      StringBuffer sql =
	new StringBuffer("SELECT t.topicid, t.num, t.creator_uid, t.top_message, t.frozen, t.archived, "
			 + "t.createdate, t.lastupdate, t.name, IFNULL(s.hidden,0) AS hidden, "
			 + "(t.top_message - IFNULL(s.last_message,-1)) AS unread FROM topics t "
			 + "LEFT JOIN topicsettings s ON t.topicid = s.topicid AND s.uid = ");
      sql.append(conf.realUID()).append(" WHERE t.confid = ").append(conf.realConfID());
      sql.append(" AND t.num = ").append(topicnum).append(';');
      if (logger.isDebugEnabled())
	logger.debug("SQL: " + sql.toString());

      // Now pass this query off to the database!
      ResultSet rs = stmt.executeQuery(sql.toString());
      if (rs.next()) // found it!
	return new TopicUserContextImpl(engine,conf,datapool,rs.getInt(1),topicnum,rs.getInt(3),
					rs.getInt(4),rs.getBoolean(5),rs.getBoolean(6),
					SQLUtil.getFullDateTime(rs,7),SQLUtil.getFullDateTime(rs,8),
					rs.getString(9),rs.getBoolean(10),rs.getInt(11));
      // else fall out and return null

    } // end try
    catch (SQLException e)
    { // turn SQLException into data exception
      logger.error("DB error reading topic entries: " + e.getMessage(),e);
      throw new DataException("unable to retrieve topic information: " + e.getMessage(),e);

    } // end catch
    finally
    { // make sure we release the connection before we go
      if (conn!=null)
	datapool.releaseConnection(conn);

    } // end finally

    return null;  // default return

  } // end getTopicByNumber

} // end class TopicUserContextImpl
